@@ -3,11 +3,9 @@ use perseus::internal::i18n::TranslationsManager;
 use perseus::internal::serve::{ServerOptions, ServerProps};
 use perseus::plugins::PluginAction;
 use perseus::stores::MutableStore;
+use perseus::PerseusApp;
 use perseus::SsrNode;
-use perseus_engine::app::{
-    get_app_root, get_error_pages_contained, get_immutable_store, get_locales, get_mutable_store,
-    get_plugins, get_static_aliases, get_templates_map_atomic_contained, get_translations_manager,
-};
+use perseus_engine as app;
 use std::env;
 use std::fs;
 
@@ -26,6 +24,7 @@ async fn main() -> std::io::Result<()> {
 
     let is_standalone = get_standalone_and_act();
     let (host, port) = get_host_and_port();
+
     HttpServer::new(move || App::new().configure(block_on(configurer(get_props(is_standalone)))))
         .bind((host, port))?
         .run()
@@ -86,7 +85,8 @@ fn get_host_and_port() -> (String, u16) {
 
 /// Gets the properties to pass to the server.
 fn get_props(is_standalone: bool) -> ServerProps<impl MutableStore, impl TranslationsManager> {
-    let plugins = get_plugins::<SsrNode>();
+    let app = app::main::<SsrNode>();
+    let plugins = app.get_plugins();
 
     plugins
         .functional_actions
@@ -95,30 +95,42 @@ fn get_props(is_standalone: bool) -> ServerProps<impl MutableStore, impl Transla
         .run((), plugins.get_plugin_data());
 
     // This allows us to operate inside `.perseus/` and as a standalone binary in production
-    let (html_shell_path, static_dir_path) = if is_standalone {
-        ("./index.html", "./static")
+    let static_dir_path = if is_standalone {
+        "./static"
     } else {
-        ("../index.html", "../static")
+        "../static"
     };
 
-    let immutable_store = get_immutable_store(&plugins);
-    let locales = get_locales(&plugins);
-    let app_root = get_app_root(&plugins);
-    let static_aliases = get_static_aliases(&plugins);
+    let immutable_store = app.get_immutable_store();
+    let locales = app.get_locales();
+    let app_root = app.get_root();
+    let static_aliases = app.get_static_aliases();
+    let templates_map = app.get_atomic_templates_map();
+    let error_pages = app.get_error_pages();
+    let index_view_str = app.get_index_view_str();
+    // Generate the global state
+    let global_state_creator = app.get_global_state_creator();
+    // By the time this binary is being run, the app has already been built be the CLI (hopefully!), so we can depend on access to hte render config
+    let index_view = block_on(PerseusApp::get_html_shell(
+        index_view_str,
+        &app_root,
+        &immutable_store,
+        &plugins,
+    ));
 
     let opts = ServerOptions {
-        // We don't support setting some attributes from `wasm-pack` through plugins/`define_app!` because that would require CLI changes as well (a job for an alternative engine)
-        index: html_shell_path.to_string(), // The user must define their own `index.html` file
+        // We don't support setting some attributes from `wasm-pack` through plugins/`PerseusApp` because that would require CLI changes as well (a job for an alternative engine)
+        html_shell: index_view,
         js_bundle: "dist/pkg/perseus_engine.js".to_string(),
         // Our crate has the same name, so this will be predictable
         wasm_bundle: "dist/pkg/perseus_engine_bg.wasm".to_string(),
-        // It's a nightmare to get the templates map to take plugins, so we use a self-contained version
-        // TODO reduce allocations here
-        templates_map: get_templates_map_atomic_contained(),
+        // This probably won't exist, but on the off chance that the user needs to support older browsers, we'll provide it anyway
+        wasm_js_bundle: "dist/pkg/perseus_engine_bg.wasm.js".to_string(),
+        templates_map,
         locales,
         root_id: app_root,
         snippets: "dist/pkg/snippets".to_string(),
-        error_pages: get_error_pages_contained(),
+        error_pages,
         // The CLI supports static content in `../static` by default if it exists
         // This will be available directly at `/.perseus/static`
         static_dir: if fs::metadata(&static_dir_path).is_ok() {
@@ -132,7 +144,8 @@ fn get_props(is_standalone: bool) -> ServerProps<impl MutableStore, impl Transla
     ServerProps {
         opts,
         immutable_store,
-        mutable_store: get_mutable_store(),
-        translations_manager: block_on(get_translations_manager()),
+        mutable_store: app.get_mutable_store(),
+        translations_manager: block_on(app.get_translations_manager()),
+        global_state_creator,
     }
 }
