@@ -8,7 +8,10 @@ use perseus_cli::{
     parse::{Opts, Subcommand},
     prepare, serve, serve_exported, tinker,
 };
-use perseus_cli::{errors::*, export_error_page, snoop_build, snoop_server, snoop_wasm_build};
+use perseus_cli::{
+    errors::*, export_error_page, order_reload, run_reload_server, snoop_build, snoop_server,
+    snoop_wasm_build,
+};
 use std::env;
 use std::io::Write;
 use std::path::PathBuf;
@@ -19,17 +22,14 @@ use std::sync::mpsc::channel;
 #[tokio::main]
 async fn main() {
     // In development, we'll test in the `basic` example
-    if cfg!(debug_assertions) {
-        let example_to_test =
-            env::var("TEST_EXAMPLE").unwrap_or_else(|_| "../../examples/basic".to_string());
+    if cfg!(debug_assertions) && env::var("TEST_EXAMPLE").is_ok() {
+        let example_to_test = env::var("TEST_EXAMPLE").unwrap();
         env::set_current_dir(example_to_test).unwrap();
     }
     let exit_code = real_main().await;
     std::process::exit(exit_code)
 }
 
-// IDEA Watch files at the `core()` level and then panic, catching the unwind in the watcher loop
-
 // This manages error handling and returns a definite exit code to terminate with
 async fn real_main() -> i32 {
     // Get the working directory
@@ -85,7 +85,7 @@ async fn core(dir: PathBuf) -> Result<i32, Error> {
     }
 
     // Parse the CLI options with `clap`
-    let opts: Opts = Opts::parse();
+    let opts = Opts::parse();
     // Check the user's environment to make sure they have prerequisites
     // We do this after any help pages or version numbers have been parsed for snappiness
     check_env()?;
@@ -113,6 +113,14 @@ async fn core(dir: PathBuf) -> Result<i32, Error> {
             })
             .expect("couldn't set handlers to gracefully terminate process");
 
+            // Set up a browser reloading server
+            // We provide an option for the user to disable this
+            if env::var("PERSEUS_NO_BROWSER_RELOAD").is_err() {
+                tokio::task::spawn(async move {
+                    run_reload_server().await;
+                });
+            }
+
             // Find out where this binary is
             // SECURITY: If the CLI were installed with root privileges, it would be possible to create a hard link to the
             // binary, execute through that, and then replace it with a malicious binary before we got here which would
@@ -153,9 +161,14 @@ async fn core(dir: PathBuf) -> Result<i32, Error> {
             // This will be updated every time we re-create the process
             // We spawn it as a process group, whcih means signals go to grandchild processes as well, which means hot reloading
             // can actually work!
-            let mut child = Command::new(&bin_name)
+            let mut child = Command::new(&bin_name);
+            let child = child
                 .args(&args)
                 .env("PERSEUS_WATCHING_PROHIBITED", "true")
+                .env("PERSEUS_USE_RELOAD_SERVER", "true"); // This is for internal use ONLY
+            #[cfg(debug_assertions)]
+            let child = child.env_remove("TEST_EXAMPLE"); // We want to use the current directory in development
+            let mut child = child
                 .group_spawn()
                 .map_err(|err| WatchError::SpawnSelfFailed { source: err })?;
 
@@ -167,9 +180,14 @@ async fn core(dir: PathBuf) -> Result<i32, Error> {
                         // This gracefully kills the process in the sense that it kills it and all its children
                         let _ = child.kill();
                         // Restart it
-                        child = Command::new(&bin_name)
+                        let mut child_l = Command::new(&bin_name);
+                        let child_l = child_l
                             .args(&args)
                             .env("PERSEUS_WATCHING_PROHIBITED", "true")
+                            .env("PERSEUS_USE_RELOAD_SERVER", "true"); // This is for internal use ONLY
+                        #[cfg(debug_assertions)]
+                        let child_l = child_l.env_remove("TEST_EXAMPLE"); // We want to use the current directory in development
+                        child = child_l
                             .group_spawn()
                             .map_err(|err| WatchError::SpawnSelfFailed { source: err })?;
                     }
@@ -194,7 +212,7 @@ async fn core(dir: PathBuf) -> Result<i32, Error> {
 async fn core_watch(dir: PathBuf, opts: Opts) -> Result<i32, Error> {
     // If we're not cleaning up artifacts, create them if needed
     if !matches!(opts.subcmd, Subcommand::Clean(_)) {
-        prepare(dir.clone())?;
+        prepare(dir.clone(), &opts.engine)?;
     }
     let exit_code = match opts.subcmd {
         Subcommand::Build(build_opts) => {
@@ -211,6 +229,8 @@ async fn core_watch(dir: PathBuf, opts: Opts) -> Result<i32, Error> {
                 return Ok(exit_code);
             }
             if export_opts.serve {
+                // Tell any connected browsers to reload
+                order_reload();
                 serve_exported(dir, export_opts.host, export_opts.port).await;
             }
             0
@@ -219,14 +239,13 @@ async fn core_watch(dir: PathBuf, opts: Opts) -> Result<i32, Error> {
             if !serve_opts.no_build {
                 delete_artifacts(dir.clone(), "static")?;
             }
+            // This orders reloads internally
             let (exit_code, _server_path) = serve(dir, serve_opts)?;
             exit_code
         }
         Subcommand::Test(test_opts) => {
             // This will be used by the subcrates
             env::set_var("PERSEUS_TESTING", "true");
-            // Set up the '.perseus/' directory if needed
-            prepare(dir.clone())?;
             // Delete old build artifacts if `--no-build` wasn't specified
             if !test_opts.no_build {
                 delete_artifacts(dir.clone(), "static")?;
@@ -271,7 +290,7 @@ async fn core_watch(dir: PathBuf, opts: Opts) -> Result<i32, Error> {
             if !tinker_opts.no_clean {
                 delete_bad_dir(dir.clone())?;
                 // Recreate the '.perseus/' directory
-                prepare(dir.clone())?;
+                prepare(dir.clone(), &opts.engine)?;
             }
             tinker(dir)?
         }
